# Generated by Django 4.2.10 on 2024-05-04 13:12

from django.db import migrations, models


class Migration(migrations.Migration):
    def delete_all_existing_process_locks(apps, schema_editor):
        ProcessLock = apps.get_model("database", "ProcessLock")
        ProcessLock.objects.all().delete()

    dependencies = [
        ("database", "0039_merge_20240501_0301"),
    ]

    operations = [
        migrations.AlterField(
            model_name="processlock",
            name="name",
            field=models.CharField(
                choices=[("index_content", "Index Content"), ("scheduled_job", "Scheduled Job")],
                max_length=200,
                unique=True,
            ),
        ),
        migrations.RunPython(delete_all_existing_process_locks, reverse_code=migrations.RunPython.noop),
    ]
