# Generated by Django 4.2.7 on 2024-01-04 12:22

import django.db.models.deletion
import phonenumber_field.modelfields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0024_alter_entry_embeddings"),
    ]

    operations = [
        migrations.CreateModel(
            name="ClientApplication",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=200)),
                ("client_id", models.CharField(max_length=200)),
                ("client_secret", models.CharField(max_length=200)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="khojuser",
            name="phone_number",
            field=phonenumber_field.modelfields.PhoneNumberField(
                blank=True, default=None, max_length=128, null=True, region=None
            ),
        ),
        migrations.AddField(
            model_name="conversation",
            name="client",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="database.clientapplication",
            ),
        ),
    ]
