# Generated by Django 4.2.10 on 2024-05-29 19:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0043_alter_chatmodeloptions_model_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="conversation",
            name="file_filters",
            field=models.JSONField(default=list),
        ),
    ]
